@@ -9,7 +9,7 @@
  * Table Visualization Widget.
  */
 qx.Class.define('callbackery.ui.plugin.Table', {
-    extend : callbackery.ui.plugin.Form,
+    extend: callbackery.ui.plugin.Form,
     /**
      * create a page for the View Tab with the given title
      *
@@ -21,23 +21,23 @@ qx.Class.define('callbackery.ui.plugin.Table', {
         }
     },
     members: {
-        _populate: function(){
+        _populate() {
             this.setLayout(new qx.ui.layout.VBox(0));
             if (this._cfg.introText) {
-                this.add(new qx.ui.basic.Label(this.xtr(this._cfg.introText)).set({rich: true}));
+                this.add(new qx.ui.basic.Label(this.xtr(this._cfg.introText)).set({ rich: true }));
             }
             this.add(this._createToolbar());
-            this.add(this._createTable(), {flex: 1});
+            this.add(this._createTable(), { flex: 1 });
         },
-        _createToolbar: function(){
+        _createToolbar() {
             var that = this;
             var cfg = this._cfg;
             var toolbar = new qx.ui.toolbar.ToolBar();
             var action = this._action = new callbackery.ui.plugin.Action(
-                cfg,qx.ui.toolbar.Button,
+                cfg, qx.ui.toolbar.Button,
                 new qx.ui.layout.HBox(0),
-                function(){
-                    if (that._form.validate()){
+                function () {
+                    if (that._form.validate()) {
                         var rpcData = that._form.getData();
                         rpcData['selection'] = that.getSelection();
                         return rpcData;
@@ -51,17 +51,23 @@ qx.Class.define('callbackery.ui.plugin.Table', {
             action.set({
                 paddingLeft: -10
             });
+            let mobileMenu = new qx.ui.form.MenuButton(this.tr("Menu"), null, action.getMobileMenu());
+            toolbar.add(mobileMenu);
+            toolbar.setOverflowIndicator(mobileMenu);
+            toolbar.setOverflowHandling(true);
+
             toolbar.add(action);
+            toolbar.setRemovePriority(action, 1);
             toolbar.addSpacer();
-            var form = this._form = new callbackery.ui.form.Auto(cfg.form,null,callbackery.ui.form.renderer.HBox, this);
+            var form = this._form = new callbackery.ui.form.Auto(cfg.form, null, callbackery.ui.form.renderer.HBox, this);
             toolbar.add(form);
-	    return toolbar;
+            return toolbar;
         },
-        _createTable: function(){
+        _createTable() {
             var cfg = this._cfg;
-            var model = this._model = new callbackery.data.RemoteTableModel(cfg,this._getParentFormData);
-            var table = this._table = new qx.ui.table.Table(model,{
-                tableColumnModel : function(obj) {
+            var model = this._model = new callbackery.data.RemoteTableModel(cfg, this._getParentFormData);
+            var table = this._table = new qx.ui.table.Table(model, {
+                tableColumnModel: function (obj) {
                     return new qx.ui.table.columnmodel.Resize(obj);
                 }
             }).set({
@@ -69,56 +75,56 @@ qx.Class.define('callbackery.ui.plugin.Table', {
             });
             table.getDataRowRenderer().setHighlightFocusRow(false);
             var ctxMenu = this._action.getTableContextMenu();
-            if (ctxMenu){
+            if (ctxMenu) {
                 table.setContextMenu(ctxMenu);
             }
             var defaultAction = this._action.getDefaultAction();
-            if (defaultAction){
-                table.addListener('cellDbltap',defaultAction,this._action);
+            if (defaultAction) {
+                table.addListener('cellDbltap', defaultAction, this._action);
             }
             var tcm = table.getTableColumnModel();
             var resizeBehavior = tcm.getBehavior();
-            cfg.table.forEach(function(col,i){
+            cfg.table.forEach(function (col, i) {
                 var cr;
                 switch (col.type) {
                     case 'boolean':
-                        cr =  new qx.ui.table.cellrenderer.Boolean;
+                        cr = new qx.ui.table.cellrenderer.Boolean;
                         break;
                     case 'html':
-                        cr =  new qx.ui.table.cellrenderer.Html;
+                        cr = new qx.ui.table.cellrenderer.Html;
                         break;
                     case 'date':
-                        cr =  new qx.ui.table.cellrenderer.Date;
+                        cr = new qx.ui.table.cellrenderer.Date;
                         if (col.format != null) {
                             cr.setDateFormat(new qx.util.format.DateFormat(col.format));
                         }
                         break;
                     case 'str':
                     case 'string':
-                        cr =  new qx.ui.table.cellrenderer.String(
-                            col.align,col.color,col.style,col.weight
+                        cr = new qx.ui.table.cellrenderer.String(
+                            col.align, col.color, col.style, col.weight
                         );
                         break;
                     case 'num':
                     case 'number':
-                        cr =  new qx.ui.table.cellrenderer.Number(
-                            col.align,col.color,col.style,col.weight
+                        cr = new qx.ui.table.cellrenderer.Number(
+                            col.align, col.color, col.style, col.weight
                         );
                         if (col.format != null) {
                             cr.setNumberFormat(
                                 new callbackery.util.format.NumberFormat()
-                                .set(col.format)
+                                    .set(col.format)
                             );
                         }
                         break;
                 }
-                if (cr){
+                if (cr) {
                     tcm.setDataCellRenderer(i, cr);
                 }
                 if ('visible' in col) {
                     tcm.setColumnVisible(i, col.visible);
                 }
-                if (col.width != null){
+                if (col.width != null) {
                     resizeBehavior.setWidth(i, String(col.width));
                 }
             });
@@ -128,10 +134,10 @@ qx.Class.define('callbackery.ui.plugin.Table', {
             selectionModel.setSelectionMode(qx.ui.table.selection.Model.SINGLE_SELECTION);
             var buttonMap = this._action.getButtonMap();
             var buttonSetMap = this._action.getButtonSetMap();
-            
-            selectionModel.addListener('changeSelection',function(){
+
+            selectionModel.addListener('changeSelection', function () {
                 var noSelection = true;
-                selectionModel.iterateSelection(function(index) {
+                selectionModel.iterateSelection(function (index) {
                     var rowData = model.getRowData(index);
                     if (rowData != null) {
                         noSelection = false;
@@ -141,54 +147,54 @@ qx.Class.define('callbackery.ui.plugin.Table', {
                             for (var key in rowData._actionSet) {
                                 var as = rowData._actionSet[key]
                                 if (as.label) {
-                                    as.label = this.xtr(as.label) 
+                                    as.label = this.xtr(as.label)
                                 }
                                 buttonMap[key].set(as);
                             }
                         }
                     }
-                },this);
-                if (noSelection){
+                }, this);
+                if (noSelection) {
                     for (var key in buttonSetMap) {
                         buttonMap[key].set(buttonSetMap[key]);
                     }
                 }
-            },this);
+            }, this);
             var processing = false;
-            model.addListener('dataChanged',function(e){
-                if (processing){
+            model.addListener('dataChanged', function (e) {
+                if (processing) {
                     return;
                 }
                 processing = true;
                 var lastData = this.getSelection();
-                if (!qx.lang.Type.isObject(lastData)){
+                if (!qx.lang.Type.isObject(lastData)) {
                     return;
                 }
-                new qx.util.DeferredCall(function(){
-                    if (currentRow !== null){
-                        for (var offset=-1;offset<=1;offset++){
-                            if (currentRow + offset < 0){
+                new qx.util.DeferredCall(function () {
+                    if (currentRow !== null) {
+                        for (var offset = -1; offset <= 1; offset++) {
+                            if (currentRow + offset < 0) {
                                 continue;
                             }
-                            var currentData = model.getRowData(currentRow+offset);
-                            if (!qx.lang.Type.isObject(currentData)){
+                            var currentData = model.getRowData(currentRow + offset);
+                            if (!qx.lang.Type.isObject(currentData)) {
                                 continue;
                             }
                             var gotPrimary = false;
                             var equal = true;
-                            cfg.table.forEach(function(col,i){
-                                if (col.primary){
+                            cfg.table.forEach(function (col, i) {
+                                if (col.primary) {
                                     gotPrimary = true;
-                                    if (currentData[col.key] != lastData[col.key]){
+                                    if (currentData[col.key] != lastData[col.key]) {
                                         equal = false;
                                     }
                                 }
                             });
-                            if (gotPrimary && equal){
-                                if (offset != 0){
+                            if (gotPrimary && equal) {
+                                if (offset != 0) {
                                     table.clearFocusedRowHighlight();
-                                    table.setFocusedCell(0,currentRow+offset,false);
-                                    selectionModel.setSelectionInterval(currentRow+offset,currentRow+offset);
+                                    table.setFocusedCell(0, currentRow + offset, false);
+                                    selectionModel.setSelectionInterval(currentRow + offset, currentRow + offset);
                                 }
                                 this.setSelection(currentData);
                                 processing = false;
@@ -200,23 +206,23 @@ qx.Class.define('callbackery.ui.plugin.Table', {
                     table.clearFocusedRowHighlight();
                     this.setSelection({});
                     processing = false;
-                },this).schedule();
-            },this);
+                }, this).schedule();
+            }, this);
 
-            this.addListener('appear',function(e){
+            this.addListener('appear', function (e) {
                 model.reloadData();
             });
-            this._form.addListener('changeData',function(e){
+            this._form.addListener('changeData', function (e) {
                 model.setFormData(e.getData());
-                if (this._loading == 0){ // only reload when data has been changed by human
+                if (this._loading == 0) { // only reload when data has been changed by human
                     this._loadDataReadOnly();
                     model.reloadData();
                 }
-            },this);
+            }, this);
 
-            this._action.addListener('actionResponse',function(e){
+            this._action.addListener('actionResponse', function (e) {
                 var data = e.getData();
-                switch (data.action){
+                switch (data.action) {
                     case 'reload':
                     case 'dataModified':
                         this._loadDataReadOnly();
